@@ -1,9 +1,7 @@
 from collections import defaultdict
-from typing import Union, List, Tuple, Dict, Set
+from typing import Dict
 from xml.etree import ElementTree
-import numpy as np
 from abc import ABC
-import warnings
 
 from commonroad import SUPPORTED_COMMONROAD_VERSIONS
 from commonroad.common.util import Interval, AngleInterval
@@ -13,7 +11,7 @@ from commonroad.planning.planning_problem import PlanningProblemSet, PlanningPro
 from commonroad.prediction.prediction import Occupancy, SetBasedPrediction, TrajectoryPrediction
 from commonroad.scenario.lanelet import Lanelet, LaneletNetwork, LineMarking, LaneletType, RoadUser, StopLine
 from commonroad.scenario.obstacle import ObstacleType, StaticObstacle, DynamicObstacle, Obstacle, EnvironmentObstacle, \
-    SignalState
+    SignalState, PhantomObstacle
 from commonroad.scenario.scenario import Scenario, Tag, GeoTransformation, Location, Environment, Time, \
     TimeOfDay, Weather, Underground, ScenarioID
 from commonroad.scenario.trajectory import State, Trajectory
@@ -24,9 +22,9 @@ from commonroad.scenario.intersection import Intersection, IntersectionIncomingE
 __author__ = "Stefanie Manzinger, Sebastian Maierhofer"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["Priority Program SPP 1835 Cooperative Interacting Automobiles", "CAR@TUM"]
-__version__ = "2020.3"
+__version__ = "2022.1"
 __maintainer__ = "Stefanie Manzinger, Sebastian Maierhofer"
-__email__ = "commonroad-i06@in.tum.de"
+__email__ = "commonroad@lists.lrz.de"
 __status__ = "Released"
 
 
@@ -207,6 +205,7 @@ class ScenarioFactory:
 
         scenario.add_objects(LaneletNetworkFactory.create_from_xml_node(xml_node))
         if commonroad_version == '2018b':
+            large_num = 10000
             scenario.add_objects(cls._obstacles_2018b(xml_node, scenario.lanelet_network, lanelet_assignment))
             for key, value in LaneletFactory._speed_limits.items():
                 for lanelet in value:
@@ -220,10 +219,13 @@ class ScenarioFactory:
                         traffic_sign_element = TrafficSignElement(TrafficSignIDSpain.MAX_SPEED, [str(key)])
                     elif SupportedTrafficSignCountry.RUSSIA.value == scenario_id.country_id:
                         traffic_sign_element = TrafficSignElement(TrafficSignIDRussia.MAX_SPEED, [str(key)])
+                    elif SupportedTrafficSignCountry.ZAMUNDA.value == scenario_id.country_id:
+                        traffic_sign_element = TrafficSignElement(TrafficSignIDZamunda.MAX_SPEED, [str(key)])
                     else:
                         traffic_sign_element = TrafficSignElement(TrafficSignIDZamunda.MAX_SPEED, [str(key)])
                         warnings.warn("Unknown country: Default traffic sign IDs are used.")
-                    traffic_sign = TrafficSign(scenario.generate_object_id(), [traffic_sign_element], {lanelet},
+                    traffic_sign = TrafficSign(scenario.generate_object_id() + large_num, [traffic_sign_element],
+                                               {lanelet},
                                                scenario.lanelet_network.find_lanelet_by_id(lanelet).right_vertices[0])
                     scenario.add_objects(traffic_sign, {lanelet})
             LaneletFactory._speed_limits = {}
@@ -269,6 +271,8 @@ class ScenarioFactory:
             obstacles.append(DynamicObstacleFactory.create_from_xml_node(o, lanelet_network, lanelet_assignment))
         for o in xml_node.findall('environmentObstacle'):
             obstacles.append(EnvironmentObstacleFactory.create_from_xml_node(o))
+        for o in xml_node.findall('phantomObstacle'):
+            obstacles.append(PhantomObstacleFactory.create_from_xml_node(o))
         return obstacles
 
 
@@ -327,12 +331,15 @@ class GeoTransformationFactory:
         :return: GeoTransformation object
         """
         geo_reference = xml_node.find('geoReference').text
-        x_translation = float(xml_node.find('xTranslation').text)
-        y_translation = float(xml_node.find('yTranslation').text)
-        z_rotation = float(xml_node.find('zRotation').text)
-        scaling = float(xml_node.find('scaling').text)
-
-        return GeoTransformation(geo_reference, x_translation, y_translation, z_rotation, scaling)
+        if xml_node.find('additionalTransformation') is not None:
+            add_trans_node = xml_node.find('additionalTransformation')
+            x_translation = float(add_trans_node.find('xTranslation').text)
+            y_translation = float(add_trans_node.find('yTranslation').text)
+            z_rotation = float(add_trans_node.find('zRotation').text)
+            scaling = float(add_trans_node.find('scaling').text)
+            return GeoTransformation(geo_reference, x_translation, y_translation, z_rotation, scaling)
+        else:
+            return GeoTransformation(geo_reference)
 
 
 class EnvironmentFactory:
@@ -1123,7 +1130,7 @@ class DynamicObstacleFactory(ObstacleFactory):
         obstacle_id = DynamicObstacleFactory.read_id(xml_node)
         shape = DynamicObstacleFactory.read_shape(xml_node.find('shape'))
         initial_state = DynamicObstacleFactory.read_initial_state(xml_node.find('initialState'))
-        initial_signal_state = StaticObstacleFactory.read_initial_signal_state(xml_node.find('initialSignalState'))
+        initial_signal_state = DynamicObstacleFactory.read_initial_signal_state(xml_node.find('initialSignalState'))
         signal_series = SignalSeriesFactory.create_from_xml_node((xml_node.find('signalSeries')))
         initial_center_lanelet_ids = set()
         initial_shape_lanelet_ids = set()
@@ -1440,11 +1447,23 @@ class PointFactory:
 
 
 class EnvironmentObstacleFactory(ObstacleFactory):
-    """ Class to create a list of objects of class Building from an XML element."""
+    """ Class to create a list of objects of type EnvironmentObstacle from an XML element."""
     @classmethod
     def create_from_xml_node(cls, xml_node: ElementTree.Element) -> EnvironmentObstacle:
         obstacle_type = EnvironmentObstacleFactory.read_type(xml_node)
-        obstacle_id = StaticObstacleFactory.read_id(xml_node)
-        shape = StaticObstacleFactory.read_shape(xml_node.find('shape'))
+        obstacle_id = EnvironmentObstacleFactory.read_id(xml_node)
+        shape = EnvironmentObstacleFactory.read_shape(xml_node.find('shape'))
 
-        return EnvironmentObstacle(obstacle_id=obstacle_id, obstacle_type=obstacle_type, obstacle_shape=shape)
\ No newline at end of file
+        return EnvironmentObstacle(obstacle_id=obstacle_id, obstacle_type=obstacle_type, obstacle_shape=shape)
+
+
+class PhantomObstacleFactory(ObstacleFactory):
+    """ Class to create a list of objects of class PhantomObstacle from an XML element."""
+    @classmethod
+    def create_from_xml_node(cls, xml_node: ElementTree.Element) -> PhantomObstacle:
+        obstacle_id = PhantomObstacleFactory.read_id(xml_node)
+        if xml_node.find('occupancySet') is not None:
+            prediction = SetBasedPredictionFactory.create_from_xml_node(xml_node.find('occupancySet'))
+        else:
+            prediction = None
+        return PhantomObstacle(obstacle_id=obstacle_id, prediction=prediction)
