@@ -7,7 +7,6 @@ import pathlib
 import os
 from typing import Union, List, Set
 import numpy as np
-import decimal
 import warnings
 
 from commonroad import SCENARIO_VERSION
@@ -20,21 +19,26 @@ from commonroad.prediction.prediction import SetBasedPrediction, TrajectoryPredi
 from commonroad.scenario.intersection import Intersection
 from commonroad.scenario.lanelet import Lanelet, LineMarking, StopLine, LaneletType
 from commonroad.scenario.obstacle import ObstacleRole, ObstacleType, DynamicObstacle, StaticObstacle, Obstacle, \
-    Occupancy, Shape, SignalState, EnvironmentObstacle
-from commonroad.scenario.scenario import Scenario, Tag, Location, GeoTransformation, Environment
-from commonroad.scenario.traffic_sign import TrafficSign, TrafficLight, TrafficLightCycleElement
+    Occupancy, Shape, SignalState, EnvironmentObstacle, PhantomObstacle
+from commonroad.scenario.scenario import Scenario, Tag, Location, GeoTransformation, Environment, Weather, TimeOfDay, \
+    Underground
+from commonroad.scenario.traffic_sign import TrafficSign, TrafficLight, TrafficLightCycleElement, TrafficLightDirection
 from commonroad.scenario.trajectory import Trajectory, State
 
 __author__ = "Stefanie Manzinger, Moritz Klischat, Sebastian Maierhofer"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["Priority Program SPP 1835 Cooperative Interacting Automobiles"]
-__version__ = "2020.3"
+__version__ = "2022.1"
 __maintainer__ = "Sebastian Maierhofer"
-__email__ = "commonroad-i06@in.tum.de"
+__email__ = "commonroad@lists.lrz.de"
 __status__ = "Released"
 
-# create a new context for this task
-ctx = decimal.Context()
+
+class DecimalPrecision:
+    decimals = 4
+
+
+precision = DecimalPrecision
 
 
 def float_to_str(f):
@@ -42,8 +46,14 @@ def float_to_str(f):
     Convert the given float to a string,
     without resorting to scientific notation
     """
-    d1 = ctx.create_decimal(repr(f))
-    return format(d1, 'f')
+    fstring = str(f)
+    if "e" in fstring:
+        return format(f, ".{}f".format(precision.decimals))
+    f_list = fstring.split(".")
+    if len(f_list) > 1:
+        return f_list[0] + "." + f_list[1][:precision.decimals]
+    else:
+        return f_list[0]
 
 
 def create_exact_node_float(value: Union[int, float]) -> etree.Element:
@@ -127,7 +137,7 @@ class CommonRoadFileWriter:
         source: str = None,
         tags: Set[Tag] = None,
         location: Location = None,
-        decimal_precision: int = 8,
+        decimal_precision: int = 4,
     ):
         """
         Initialize the FileWriter with a scenario and tags for the xml-header
@@ -155,9 +165,8 @@ class CommonRoadFileWriter:
         self.location = location if location is not None else scenario.location
         self.tags = tags if tags is not None else scenario.tags
 
-
         # set decimal precision
-        ctx.prec = decimal_precision
+        precision.decimals = decimal_precision
 
     @property
     def root_node(self):
@@ -251,8 +260,6 @@ class CommonRoadFileWriter:
             self._root_node.append(IntersectionXMLNode.create_node(intersection))
         for o in self.scenario.obstacles:
             self._root_node.append(ObstacleXMLNode.create_node(o))
-        for b in self.scenario.environment_obstacle:
-            self._root_node.append(EnvironmentObstacleXMLNode.create_node(b))
 
     def _add_all_planning_problems_from_planning_problem_set(self):
         for (
@@ -282,7 +289,7 @@ class CommonRoadFileWriter:
         :return:
         """
         if filename is None:
-            filename = str(self.scenario.scenario_id)
+            filename = str(self.scenario.scenario_id) + ".xml"
 
         if pathlib.Path(filename).is_file():
             if overwrite_existing_file is OverwriteExistingFile.ASK_USER_INPUT:
@@ -296,7 +303,7 @@ class CommonRoadFileWriter:
             else:
                 overwrite = 'y'
 
-            if overwrite is 'n':
+            if overwrite == 'n':
                 print('Writing of file {} skipped'.format(filename))
                 return
             else:
@@ -306,7 +313,7 @@ class CommonRoadFileWriter:
         self._add_all_objects_from_scenario()
         self._add_all_planning_problems_from_planning_problem_set()
         if check_validity:
-            # validate xml format 
+            # validate xml format
             self.check_validity_of_commonroad_file(self._dump())
 
         tree = etree.ElementTree(self._root_node)
@@ -321,7 +328,7 @@ class CommonRoadFileWriter:
         Write a scenario without planning-problem. If file already exists, it will be overwritten of skipped.
 
         :param filename: filename of the xml output file. If 'None', the Benchmark ID is taken
-        :param OverwriteExistingFile: Specify whether an already existing file should be overwritten or skipped
+        :param overwrite_existing_file: Specify whether an already existing file should be overwritten or skipped
         :return: None
         """
         if filename is None:
@@ -339,7 +346,7 @@ class CommonRoadFileWriter:
             else:
                 overwrite = 'y'
 
-            if overwrite is 'n':
+            if overwrite == 'n':
                 print(
                     'Writing skipped for file, since it already exists {}'.format(
                         filename
@@ -366,7 +373,7 @@ class CommonRoadFileWriter:
 
         """
         with open(
-            os.path.dirname(os.path.abspath(__file__)) + '/XML_commonRoad_XSD.xsd',
+            os.path.dirname(os.path.abspath(__file__)) + '/../xml_definition_files/XML_commonRoad_XSD.xsd',
             'rb',
         ) as schema_file:
             schema = etree.XMLSchema(etree.parse(schema_file))
@@ -446,21 +453,24 @@ class EnvironmentXMLNode:
         :return: node
         """
         environment_node = etree.Element('environment')
-        time_node = etree.Element('time')
-        if environment.time.hours < 10:
-            time_node.text = "0" + str(environment.time.hours) + ":" + str(environment.time.minutes) + ":00"
-        else:
-            time_node.text = str(environment.time.hours) + ":" + str(environment.time.minutes)
-        environment_node.append(time_node)
-        time_of_day_node = etree.Element('timeOfDay')
-        time_of_day_node.text = environment.time_of_day.value
-        environment_node.append(time_of_day_node)
-        weather_node = etree.Element('weather')
-        weather_node.text = environment.weather.value
-        environment_node.append(weather_node)
-        underground_node = etree.Element('underground')
-        underground_node.text = environment.underground.value
-        environment_node.append(underground_node)
+        if environment.time_of_day.value is not TimeOfDay.UNKNOWN:
+            time_node = etree.Element('time')
+            if environment.time.hours < 10:
+                time_node.text = "0" + str(environment.time.hours) + ":" + str(environment.time.minutes) + ":00"
+            else:
+                time_node.text = str(environment.time.hours) + ":" + str(environment.time.minutes)
+            environment_node.append(time_node)
+            time_of_day_node = etree.Element('timeOfDay')
+            time_of_day_node.text = environment.time_of_day.value
+            environment_node.append(time_of_day_node)
+        if environment.weather.value is not Weather.UNKNOWN:
+            weather_node = etree.Element('weather')
+            weather_node.text = environment.weather.value
+            environment_node.append(weather_node)
+        if environment.underground.value is not Underground.UNKNOWN:
+            underground_node = etree.Element('underground')
+            underground_node.text = environment.underground.value
+            environment_node.append(underground_node)
 
         return environment_node
 
@@ -499,7 +509,7 @@ class LaneletXMLNode:
 
         if hasattr(lanelet, 'line_marking_left_vertices') and isinstance(
             lanelet.line_marking_left_vertices, LineMarking
-        ):
+        ) and lanelet.line_marking_left_vertices is not LineMarking.UNKNOWN:
             line_marking_left = etree.Element('lineMarking')
             line_marking_left.text = lanelet.line_marking_left_vertices.value
             left_boundary.append(line_marking_left)
@@ -514,21 +524,21 @@ class LaneletXMLNode:
 
         if hasattr(lanelet, 'line_marking_right_vertices') and isinstance(
             lanelet.line_marking_right_vertices, LineMarking
-        ):
+        ) and lanelet.line_marking_right_vertices is not LineMarking.UNKNOWN:
             line_marking_right = etree.Element('lineMarking')
             line_marking_right.text = lanelet.line_marking_right_vertices.value
             right_boundary.append(line_marking_right)
 
         lanelet_node.append(right_boundary)
 
-        for l in lanelet.predecessor:
+        for la in lanelet.predecessor:
             predecessor = etree.Element('predecessor')
-            predecessor.set('ref', str(l))
+            predecessor.set('ref', str(la))
             lanelet_node.append(predecessor)
 
-        for l in lanelet.successor:
+        for la in lanelet.successor:
             successor = etree.Element('successor')
-            successor.set('ref', str(l))
+            successor.set('ref', str(la))
             lanelet_node.append(successor)
 
         if lanelet.adj_left:
@@ -592,7 +602,8 @@ class LaneletXMLNode:
 
 class ObstacleXMLNode:
     @classmethod
-    def create_node(cls, obstacle: Union[Obstacle, DynamicObstacle, StaticObstacle]) -> etree.Element:
+    def create_node(cls, obstacle: Union[Obstacle, DynamicObstacle, StaticObstacle, EnvironmentObstacle,
+                                         PhantomObstacle]) -> etree.Element:
         """
         Create XML-Node for an Obstacle
         :param obstacle: Obstacle for creating a node
@@ -602,8 +613,10 @@ class ObstacleXMLNode:
             return DynamicObstacleXMLNode.create_node(obstacle)
         elif type(obstacle) == StaticObstacle:
             return StaticObstacleXMLNode.create_node(obstacle)
-        elif type(obstacle == EnvironmentObstacle):
+        elif type(obstacle) == EnvironmentObstacle:
             return EnvironmentObstacleXMLNode.create_node(obstacle)
+        elif type(obstacle) == PhantomObstacle:
+            return PhantomObstacleXMLNode.create_node(obstacle)
         else:
             raise Exception()
 
@@ -613,9 +626,6 @@ class ObstacleXMLNode:
     ):
         obstacle_node = etree.Element(obstacle_role.value+'Obstacle')
         obstacle_node.set('id', str(obstacle_id))
-        # role_node = etree.Element('role')
-        # role_node.text = cls._obstacle_role_enum_to_string(obstacle_role)
-        # obstacle_node.append(role_node)
         type_node = etree.Element('type')
         type_node.text = obstacle_type.value
         obstacle_node.append(type_node)
@@ -627,7 +637,7 @@ class EnvironmentObstacleXMLNode:
     def create_node(cls, environment_obstacle: EnvironmentObstacle) -> etree.Element:
         """
         Create XML-Node for a EnvironmentObstacle
-        :param environment_obstacle: environment_obstacle for creating a node
+        :param environment_obstacle: EnvironmentObstacle for creating a node
         :return: node
         """
         node = ObstacleXMLNode.create_obstacle_node_header(
@@ -642,6 +652,27 @@ class EnvironmentObstacleXMLNode:
         return node
 
 
+class PhantomObstacleXMLNode:
+    @classmethod
+    def create_node(cls, phantom_obstacle: PhantomObstacle) -> etree.Element:
+        """
+        Create XML-Node for a PhantomObstacle
+        :param phantom_obstacle: PhantomObstacle for creating a node
+        :return: node
+        """
+        node = PhantomObstacleXMLNode.create_obstacle_node_header(phantom_obstacle.obstacle_id,
+                                                                  phantom_obstacle.obstacle_role)
+        if isinstance(phantom_obstacle.prediction, SetBasedPrediction):
+            node.append(DynamicObstacleXMLNode.create_occupancy_node(phantom_obstacle.prediction.occupancy_set))
+        return node
+
+    @classmethod
+    def create_obstacle_node_header(cls, obstacle_id: int, obstacle_role: ObstacleRole):
+        obstacle_node = etree.Element(obstacle_role.value+'Obstacle')
+        obstacle_node.set('id', str(obstacle_id))
+        return obstacle_node
+
+
 class StaticObstacleXMLNode:
     @classmethod
     def create_node(cls, static_obstacle: StaticObstacle) -> etree.Element:
@@ -714,7 +745,7 @@ class DynamicObstacleXMLNode:
         # write prediction depending on type
         if isinstance(dynamic_obstacle.prediction, SetBasedPrediction):
             obstacle_node.append(
-                cls._create_occupancy_node(dynamic_obstacle.prediction.occupancy_set)
+                cls.create_occupancy_node(dynamic_obstacle.prediction.occupancy_set)
             )
         elif isinstance(dynamic_obstacle.prediction, TrajectoryPrediction):
             obstacle_node.append(
@@ -743,7 +774,7 @@ class DynamicObstacleXMLNode:
         return traj_node
 
     @classmethod
-    def _create_occupancy_node(cls, occupancy_set: List[Occupancy]) -> etree.Element:
+    def create_occupancy_node(cls, occupancy_set: List[Occupancy]) -> etree.Element:
         """
         Create XML-Node for an occupancy_set
         :param occupancy_set: occupancy_set for creating a node
@@ -758,7 +789,7 @@ class DynamicObstacleXMLNode:
     def _create_signal_series_node(cls, signal_series: List[SignalState]) -> etree.Element:
         """
         Create XML-Node for a Trajectory
-        :param trajectory: trajectory for creating a node
+        :param signal_series: list of signal states
         :return: node
         """
         series_node = etree.Element('signalSeries')
@@ -817,7 +848,7 @@ class ShapeXMLNode:
 
     @classmethod
     def _create_single_element(
-        cls, shape: Shape, dynamic_obstacle_shape: bool
+        cls, shape: Union[Shape, Circle, Rectangle, Polygon], dynamic_obstacle_shape: bool
     ) -> etree.Element:
         """
         Create XML-Node for a single shape element
@@ -866,10 +897,10 @@ class RectangleXMLNode:
 
             center_node = etree.Element('center')
             x_node = etree.Element('x')
-            x_node.text = str(np.float64(rectangle.center[0]))
+            x_node.text = float_to_str(np.float64(rectangle.center[0]))
             center_node.append(x_node)
             y_node = etree.Element('y')
-            y_node.text = str(np.float64(rectangle.center[1]))
+            y_node.text = float_to_str(np.float64(rectangle.center[1]))
             center_node.append(y_node)
             rectangle_node.append(center_node)
         return rectangle_node
@@ -895,10 +926,10 @@ class CircleXMLNode:
         if not dynamic_obstacle_shape:
             center_node = etree.Element('center')
             x_node = etree.Element('x')
-            x_node.text = str(np.float64(circle.center[0]))
+            x_node.text = float_to_str(np.float64(circle.center[0]))
             center_node.append(x_node)
             y_node = etree.Element('y')
-            y_node.text = str(np.float64(circle.center[1]))
+            y_node.text = float_to_str(np.float64(circle.center[1]))
             center_node.append(y_node)
             circle_node.append(center_node)
         return circle_node
@@ -979,19 +1010,15 @@ class StateXMLNode:
         :return: node
         """
         if len(goal_lanelet_ids) > 0:
-            for id in goal_lanelet_ids:
+            for la_id in goal_lanelet_ids:
                 lanelet = etree.Element('lanelet')
-                lanelet.set('ref', str(id))
+                lanelet.set('ref', str(la_id))
                 node.append(lanelet)
         elif isinstance(position, int):
             lanelet = etree.Element('lanelet')
             lanelet.set('ref', str(position))
             node.append(lanelet)
-        elif(
-            isinstance(position, Rectangle)
-            or isinstance(position, Circle)
-            or isinstance(position, Polygon)
-            ):
+        elif isinstance(position, Rectangle) or isinstance(position, Circle) or isinstance(position, Polygon):
             node.extend(ShapeXMLNode.create_node(position))
         elif isinstance(position, ShapeGroup):
             node.extend(ShapeXMLNode.create_node(position))
@@ -1048,6 +1075,7 @@ class StateXMLNode:
         Create XML-Node for a state
         :param state: value of the state
         :param state_node: node of the overlying state
+        :param time_step: time step for which state should be created
         :return: node
         """
 
@@ -1298,10 +1326,17 @@ class TrafficLightXMLNode:
             position_node.append(Point(traffic_light.position[0],
                                        traffic_light.position[1]).create_node())
             traffic_light_node.append(position_node)
+
+        if traffic_light.direction is not TrafficLightDirection.ALL:
+            direction_node = etree.Element('direction')
+            direction_node.text = traffic_light.direction.value
+            traffic_light_node.append(direction_node)
+
         if traffic_light.active is not None:
             active_node = etree.Element('active')
             active_node.text = str(traffic_light.active).lower()
             traffic_light_node.append(active_node)
+
         return traffic_light_node
 
     @classmethod
@@ -1331,7 +1366,7 @@ class LineMarkingXMLNode:
         return str(line_marking.name.lower())
 
     @classmethod
-    def create_node(cls, line_marking) -> etree.Element:
+    def create_node(cls, line_marking: LineMarking) -> etree.Element:
         line_marking_node = etree.Element('lineMarking')
         line_marking_node.text = cls._line_marking_enum_to_string(line_marking)
         return line_marking_node
@@ -1396,7 +1431,7 @@ class SignalStateXMLNode:
 
         if hasattr(signal_state, 'hazard_warning_lights'):
             hazard_warning_lights = etree.Element('hazardWarningLights')
-            hazard_warning_lights.text = str(signal_state.braking_lights).lower()
+            hazard_warning_lights.text = str(signal_state.hazard_warning_lights).lower()
             signal_state_node.append(hazard_warning_lights)
 
         if hasattr(signal_state, 'flashing_blue_lights'):
