@@ -7,19 +7,25 @@ from xml.dom import minidom
 import numpy as np
 import xml.etree.ElementTree as et
 from enum import Enum, unique
-from typing import List, Tuple, Union
+from typing import List, Tuple, Union, Dict
 from datetime import datetime
+import vehiclemodels.parameters_vehicle1 as p1
+import vehiclemodels.parameters_vehicle2 as p2
+import vehiclemodels.parameters_vehicle3 as p3
 
 from commonroad.common.validity import is_real_number, is_positive
+from commonroad.geometry.shape import Rectangle
+from commonroad.prediction.prediction import TrajectoryPrediction
+from commonroad.scenario.obstacle import DynamicObstacle, ObstacleType
 from commonroad.scenario.scenario import ScenarioID
 from commonroad.scenario.trajectory import State, Trajectory
 
 __author__ = "Murat Üste, Christina Miller, Moritz Klischat"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["BMW CAR@TUM"]
-__version__ = "2020.3"
+__version__ = "2022.1"
 __maintainer__ = "Moritz Klischat"
-__email__ = "commonroad-i06@in.tum.de"
+__email__ = "commonroad@lists.lrz.de"
 __status__ = "Released"
 
 
@@ -51,6 +57,11 @@ class VehicleType(Enum):
     VW_VANAGON = 3
 
 
+vehicle_parameters = {VehicleType.FORD_ESCORT: p1.parameters_vehicle1(),
+                      VehicleType.BMW_320i: p2.parameters_vehicle2(),
+                      VehicleType.VW_VANAGON: p3.parameters_vehicle3()}
+
+
 @unique
 class VehicleModel(Enum):
     PM = 0
@@ -67,6 +78,8 @@ class CostFunction(Enum):
     SM1 = 3
     SM2 = 4
     SM3 = 5
+    MW1 = 6
+    TR1 = 7
 
 
 @unique
@@ -170,25 +183,30 @@ class StateType(Enum):
             state_fields_all = [StateFields[str(desired_vehicle_model.name)], StateFields.Input, StateFields.PMInput]
             state_fields_add = []
             for sf in StateFields:
-                if sf not in state_fields_all: state_fields_add.append(sf)
+                if sf not in state_fields_all:
+                    state_fields_add.append(sf)
 
             state_fields_all += state_fields_add
 
             for state_fields in state_fields_all:
-                if not len(attrs) >= len(state_fields.value): continue  # >=
-                if not all([sf in attrs for sf in state_fields.value]): continue
+                if not len(attrs) >= len(state_fields.value):
+                    continue  # >=
+                if not all([sf in attrs for sf in state_fields.value]):
+                    continue
                 return cls[state_fields.name]
         else:
             state_fields_all = StateFields
             for state_fields in state_fields_all:
-                if not len(attrs) == len(state_fields.value): continue  # ==
-                if not all([sf in attrs for sf in state_fields.value]): continue
+                if not len(attrs) == len(state_fields.value):
+                    continue  # ==
+                if not all([sf in attrs for sf in state_fields.value]):
+                    continue
                 return cls[state_fields.name]
 
         raise StateTypeException('Given state is not valid!')
 
     @classmethod
-    def check_state_type(cls, vehicle_model: VehicleModel) -> bool:
+    def check_state_type(cls, vehicle_model: VehicleModel) -> None:
         """
         Checks whether vehicle model can be supported by trajectory.
         :param vehicle_model: vehicle model enum
@@ -253,7 +271,7 @@ class SupportedCostFunctions(Enum):
     """
     Enum class for specifying which cost functions are supported for which vehicle model
     """
-    PM = [CostFunction.JB1, CostFunction.WX1]
+    PM = [CostFunction.JB1, CostFunction.WX1, CostFunction.MW1]
     ST = [cost_function for cost_function in CostFunction]  # Supports all cost functions
     KS = [cost_function for cost_function in CostFunction]  # Supports all cost functions
     MB = [cost_function for cost_function in CostFunction]  # Supports all cost functions
@@ -301,8 +319,7 @@ class PlanningProblemSolution:
                                                                                         vehicle_model.name))
         return True
 
-    @staticmethod
-    def _check_trajectory_supported(vehicle_model: VehicleModel, trajectory_type: TrajectoryType) -> bool:
+    def _check_trajectory_supported(self, vehicle_model: VehicleModel, trajectory_type: TrajectoryType) -> bool:
         """
         Checks whether given vehicle model is valid for the given trajectory type.
 
@@ -310,6 +327,11 @@ class PlanningProblemSolution:
         :param trajectory_type: TrajectoryType
         :return: True if valid.
         """
+        if self._vehicle_model == VehicleModel.PM and self._trajectory_type == TrajectoryType.PM:
+            for state in self._trajectory.state_list:
+                if not hasattr(state, 'orientation'):
+                    state.orientation = math.atan2(state.velocity_y, state.velocity)
+
         if not trajectory_type.valid_vehicle_model(vehicle_model):
             raise SolutionException('Vehicle model %s is not valid for the trajectory type %s!'
                                     % (vehicle_model.name, trajectory_type.name))
@@ -405,12 +427,21 @@ class Solution:
             Default=None.
         """
         self.scenario_id = scenario_id
+        self._planning_problem_solutions: Dict[int, PlanningProblemSolution] = {}
         self.planning_problem_solutions = planning_problem_solutions
         self.date = date
         self._computation_time = None
         self.computation_time = computation_time
         self.processor_name = processor_name
 
+    @property
+    def planning_problem_solutions(self) -> List[PlanningProblemSolution]:
+        return list(self._planning_problem_solutions.values())
+
+    @planning_problem_solutions.setter
+    def planning_problem_solutions(self, planning_problem_solutions: List[PlanningProblemSolution]):
+        self._planning_problem_solutions = {s.planning_problem_id: s for s in planning_problem_solutions}
+
     @property
     def benchmark_id(self) -> str:
         """
@@ -509,6 +540,7 @@ class Solution:
     def computation_time(self) -> Union[None, float]:
         """
         Return the computation time [s] for the trajectory.
+
         :return:
         """
         return self._computation_time
@@ -523,6 +555,26 @@ class Solution:
                 .format(type(computation_time))
         self._computation_time = computation_time
 
+    def create_dynamic_obstacle(self) -> Dict[int, DynamicObstacle]:
+        """
+        Creates dynamic obstacle(s) from solution(s) for every planning problem.
+        :return:
+        """
+        obs = {}
+        for pp_id, solution in self._planning_problem_solutions.items():
+            shape = Rectangle(length=vehicle_parameters[solution.vehicle_type].l,
+                              width=vehicle_parameters[solution.vehicle_type].w)
+            trajectory = Trajectory(initial_time_step=solution.trajectory.initial_time_step + 1,
+                                    state_list=solution.trajectory.state_list[1:])
+            prediction = TrajectoryPrediction(trajectory, shape=shape)
+            obs[pp_id] = DynamicObstacle(obstacle_id=pp_id,
+                                         obstacle_type=ObstacleType.CAR,
+                                         obstacle_shape=shape,
+                                         initial_state=solution.trajectory.state_list[0],
+                                         prediction=prediction)
+
+        return obs
+
 
 class CommonRoadSolutionReader:
     """Reads solution xml files created with the CommonRoadSolutionWriter"""
@@ -564,13 +616,17 @@ class CommonRoadSolutionReader:
     @staticmethod
     def _parse_header(root_node: et.Element) -> Tuple[str, Union[None, datetime], Union[None, float], Union[None, str]]:
         """ Parses the header attributes for the given Solution XML root node. """
-        benchmark_id = root_node.get('benchmark_id', None)
+        benchmark_id = root_node.get('benchmark_id')
         if not benchmark_id:
             SolutionException("Solution xml does not have a benchmark id!")
 
         date = root_node.attrib.get('date', None)  # None if not found
         if date is not None:
-            date = datetime.strptime(date, '%Y-%m-%d')
+            try:
+                date = datetime.strptime(date, '%Y-%m-%dT%H:%M:%S')
+            except ValueError:
+                # backward compatibility with old solution files
+                date = datetime.strptime(date, '%Y-%m-%d')
 
         computation_time = root_node.attrib.get('computation_time', None)
         if computation_time is not None:
@@ -630,9 +686,6 @@ class CommonRoadSolutionReader:
             else:
                 state_vals[field_name] = cls._parse_sub_element(state_node, xml_name, as_float=(not xml_name == 'time'))
 
-            if not 'orientation' in state_vals and ('velocity' in state_vals and 'velocity_y' in state_vals):
-                state_vals['orientation'] = math.atan2(state_vals['velocity_y'], state_vals['velocity'])
-
         return State(**state_vals)
 
     @staticmethod
@@ -683,8 +736,8 @@ class CommonRoadSolutionWriter:
         delete_from_cpu_name = ['(R)', '(TM)']
 
         def strip_substrings(string: str):
-            for del_str in delete_from_cpu_name:
-                string = string.replace(del_str, '')
+            for del_string in delete_from_cpu_name:
+                string = string.replace(del_string, '')
             return string
 
         if platform.system() == "Windows":
@@ -721,10 +774,13 @@ class CommonRoadSolutionWriter:
         """ Creates the root node of the Solution XML. """
         root_node = et.Element('CommonRoadSolution')
         root_node.set('benchmark_id', solution.benchmark_id)
-        if solution.date is not None: root_node.set('date', solution.date.strftime('%Y-%m-%d'))
-        if solution.computation_time is not None: root_node.set('computation_time', str(solution.computation_time))
+        if solution.computation_time is not None:
+            root_node.set('computation_time', str(solution.computation_time))
+        if solution.date is not None:
+            root_node.set('date', solution.date.strftime('%Y-%m-%dT%H:%M:%S'))
         processor_name = cls._get_processor_name() if solution.processor_name == 'auto' else solution.processor_name
-        if processor_name is not None: root_node.set('processor_name', processor_name)
+        if processor_name is not None:
+            root_node.set('processor_name', processor_name)
         return root_node
 
     @classmethod
