@@ -1,6 +1,6 @@
 import copy
 import math
-from typing import Union, List, Dict, Set
+from typing import Union, List, Dict, Set, Tuple, Optional
 import numpy as np
 import warnings
 
@@ -11,22 +11,32 @@ from commonroad.scenario.trajectory import State
 __author__ = "Christina Miller and Stefanie Manzinger"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["Priority Program SPP 1835 Cooperative Interacting Automobiles"]
-__version__ = "2020.3"
+__version__ = "2022.1"
 __maintainer__ = "Christina Miller"
-__email__ = "commonroad-i06@in.tum.de"
+__email__ = "commonroad@lists.lrz.de"
 __status__ = "Released"
 
+from commonroad.visualization.drawable import IDrawable
+from commonroad.visualization.param_server import ParamServer
+from commonroad.visualization.renderer import IRenderer
 
-class GoalRegion:
+
+class GoalRegion(IDrawable):
     def __init__(self, state_list: List[State],
-                 lanelets_of_goal_position: Union[None, Dict[int, List[int]]]=None):
+                 lanelets_of_goal_position: Union[
+                     None, Dict[int, List[int]]] = None):
         """
-        Region, that has to be reached by the vehicle. Contains a list of goal states of which one has to be fulfilled
-        to solve the scenario. If 'position' in a goal state is given as a list of lanelets, they are converted into a
-        polygon. To reconstruct the lanelets later, the lanelet ids are stored in a dict in lanelets_of_goal_position.
-        In no 'position' is given as lanelet, lanelets_of_goal_position is set to None.
+        Region, that has to be reached by the vehicle. Contains a list of
+        goal states of which one has to be fulfilled
+        to solve the scenario. If 'position' in a goal state is given as a
+        list of lanelets, they are converted into a
+        polygon. To reconstruct the lanelets later, the lanelet ids are
+        stored in a dict in lanelets_of_goal_position.
+        In no 'position' is given as lanelet, lanelets_of_goal_position is
+        set to None.
 
-        :param state_list: list of goal states (one of those has to be fulfilled)
+        :param state_list: list of goal states (one of those has to be
+        fulfilled)
         :param lanelets_of_goal_position: dict[index of state in state_list, list of lanelet ids].
         None, if no lanelet is given.
         """
@@ -76,7 +86,7 @@ class GoalRegion:
             goal_state_fields = set([slot for slot in goal_state.__slots__ if hasattr(goal_state, slot)])
             state_fields = set([slot for slot in goal_state.__slots__ if hasattr(state, slot)])
             state_new, state_fields, goal_state_tmp, goal_state_fields =\
-                self._harmonize_state_types(state,goal_state_tmp, state_fields, goal_state_fields)
+                self._harmonize_state_types(state, goal_state_tmp, state_fields, goal_state_fields)
 
             if not goal_state_fields.issubset(state_fields):
                 raise ValueError('The goal states {} are not a subset of the provided states {}!'
@@ -156,7 +166,9 @@ class GoalRegion:
                                      '%s only (except from position and orientation); got "%s" for '
                                      'attribute "%s"' % (Interval, getattr(state, attr).__class__, attr))
 
-    def _harmonize_state_types(self, state:State, goal_state: State,  state_fields: Set[str], goal_state_fields: Set[str]):
+    @staticmethod
+    def _harmonize_state_types(state: State, goal_state: State,  state_fields: Set[str],
+                               goal_state_fields: Set[str]):
         """
         Transforms states from value_x, value_y to orientation, value representation if required.
         :param state: state to check for goal
@@ -165,14 +177,21 @@ class GoalRegion:
         """
         state_new = copy.deepcopy(state)
         if {'velocity', 'velocity_y'}.issubset(state_fields) \
-            and {'orientation'}.issubset(goal_state_fields) \
-            and not {'velocity', 'velocity_y'}.issubset(goal_state_fields):
+                and ({'orientation'}.issubset(goal_state_fields) or {'velocity'}.issubset(goal_state_fields))\
+                and not {'velocity', 'velocity_y'}.issubset(goal_state_fields):
 
-            if not 'orientation' in state_fields:
-                state_new.orientation = math.atan2(state_new.velocity_y, state_new.velocity)
+            if 'orientation' not in state_fields:
+                state_new.orientation = math.atan2(state_new.velocity_y,
+                                                   state_new.velocity)
                 state_fields.add('orientation')
 
-            state_new.velocity = np.linalg.norm(np.array([state_new.velocity, state_new.velocity_y]))
+            state_new.velocity = np.linalg.norm(
+                    np.array([state_new.velocity, state_new.velocity_y]))
             state_fields.remove('velocity_y')
 
         return state_new, state_fields, goal_state, goal_state_fields
+
+    def draw(self, renderer: IRenderer,
+             draw_params: Union[ParamServer, dict, None] = None,
+             call_stack: Optional[Tuple[str, ...]] = tuple()):
+        renderer.draw_goal_region(self, draw_params, call_stack)
