@@ -1,9 +1,9 @@
 __author__ = "Sebastian Maierhofer"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["CAR@TUM"]
-__version__ = "2020.3"
+__version__ = "2022.1"
 __maintainer__ = "Sebastian Maierhofer"
-__email__ = "commonroad-i06@in.tum.de"
+__email__ = "commonroad@lists.lrz.de"
 __status__ = "Release"
 
 from typing import List, Set, Dict
@@ -20,9 +20,9 @@ class IntersectionIncomingElement:
     is valid for and to facilitate the calculation of priorities at intersections.
     The left incoming is used to infer the right-before-left-rule.
     """
+
     def __init__(self, incoming_id: int, incoming_lanelets: Set[int] = None, successors_right: Set[int] = None,
-                 successors_straight: Set[int] = None, successors_left: Set[int] = None,
-                 left_of: int = None):
+                 successors_straight: Set[int] = None, successors_left: Set[int] = None, left_of: int = None):
         """
         :param incoming_id: incoming element ID
         :param incoming_lanelets: set of IDs of incoming lanelets
@@ -38,7 +38,37 @@ class IntersectionIncomingElement:
         self.successors_right = successors_right
         self.successors_straight = successors_straight
         self.successors_left = successors_left
-        self.left_of = left_of
+        self._left_of = left_of
+
+    def __eq__(self, other):
+        if not isinstance(other, IntersectionIncomingElement):
+            warnings.warn(f"Inequality between IntersectionIncomingElement "
+                          f"{repr(self)} and different type {type(other)}")
+            return False
+
+        if self._incoming_id == other.incoming_id and self._incoming_lanelets == other.incoming_lanelets \
+                and self._successors_right == other.successors_right \
+                and self._successors_straight == other.successors_straight \
+                and self._successors_left == other.successors_left and self._left_of == other.left_of:
+            return True
+
+        warnings.warn(f"Inequality of IntersectionIncomingElement {repr(self)} and the other one {repr(other)}")
+        return False
+
+    def __hash__(self):
+        return hash((self._incoming_id, frozenset(self._incoming_lanelets), frozenset(self._successors_right),
+                     frozenset(self._successors_straight), frozenset(self._successors_left), self._left_of))
+
+    def __str__(self):
+        return f"IntersectionIncomingElement with id {self._incoming_id} represents the incoming " \
+               f"lanelets {self._incoming_lanelets} and has right successors {self._successors_right}, " \
+               f"straight successors {self._successors_straight}, and left successors {self._successors_left}"
+
+    def __repr__(self):
+        return f"IntersectionIncomingElement(incoming_id={self._incoming_id}, " \
+               f"incoming_lanelets={self._incoming_lanelets}, successors_right={self._successors_right}, " \
+               f"successors_straight={self._successors_straight}, successors_left={self._successors_left}, " \
+               f"left_of={self._left_of})"
 
     @property
     def incoming_id(self) -> int:
@@ -91,7 +121,10 @@ class IntersectionIncomingElement:
         """
         :param successors_right: set of IDs of incoming lanelets which turn right
         """
-        self._successors_right = successors_right
+        if successors_right is None:
+            self._successors_right = set()
+        else:
+            self._successors_right = successors_right
 
     @property
     def successors_straight(self) -> Set[int]:
@@ -105,7 +138,10 @@ class IntersectionIncomingElement:
         """
         :param successors_straight: set of IDs of incoming lanelets which go straight
         """
-        self._successors_straight = successors_straight
+        if successors_straight is None:
+            self._successors_straight = set()
+        else:
+            self._successors_straight = successors_straight
 
     @property
     def successors_left(self) -> Set[int]:
@@ -119,7 +155,10 @@ class IntersectionIncomingElement:
         """
         :param successors_left: set of IDs of incoming lanelets which turn left
         """
-        self._successors_left = successors_left
+        if successors_left is None:
+            self._successors_left = set()
+        else:
+            self._successors_left = successors_left
 
     @property
     def left_of(self) -> int:
@@ -142,6 +181,7 @@ class Intersection:
     An intersection element is defined by at least one incoming and an optional crossing element.
     The crossing element models lanelets which cross other lanelets, e.g., these are usually lanelets of type crosswalk.
     """
+
     def __init__(self, intersection_id: int, incomings: List[IntersectionIncomingElement], crossings: Set[int] = None):
         """
         :param intersection_id: ID of intersection element
@@ -156,6 +196,39 @@ class Intersection:
         self.incomings = incomings
         self.crossings = crossings
 
+    def __eq__(self, other):
+        if not isinstance(other, Intersection):
+            warnings.warn(f"Inequality between Intersection {repr(self)} and different type {type(other)}")
+            return False
+
+        list_elements_eq = True
+        incomings = {incoming.incoming_id: incoming for incoming in self._incomings}
+        incomings_other = {incoming.incoming_id: incoming for incoming in other.incomings}
+        intersection_eq = len(incomings) == len(incomings_other)
+        for k in incomings.keys():
+            if k not in incomings_other:
+                intersection_eq = False
+                continue
+            if incomings.get(k) != incomings_other.get(k):
+                list_elements_eq = False
+
+        if intersection_eq and self._intersection_id == other.intersection_id and self._crossings == other.crossings:
+            return list_elements_eq
+
+        warnings.warn(f"Inequality of Intersection {repr(self)} and the other one {repr(other)}")
+        return False
+
+    def __hash__(self):
+        return hash((self._intersection_id, frozenset(self._incomings), frozenset(self._crossings)))
+
+    def __str__(self):
+        return f"Intersection with id {self._intersection_id} consisting of {len(self._incomings)} incoming elements " \
+               f"and crossings {self._crossings}"
+
+    def __repr__(self):
+        return f"Intersection(intersection_id={self._intersection_id}, incomings={repr(self._incomings)}, " \
+               f"crossings={self._crossings})"
+
     @property
     def intersection_id(self) -> int:
         """
@@ -189,7 +262,7 @@ class Intersection:
         """
         if self._incomings is None:
             assert len(incomings) > 0, '<Intersection/incomings>: Intersection ' \
-                                               'must consist of at least two incomings '
+                                       'must consist of at least two incomings '
             self._incomings = incomings
         else:
             warnings.warn('<Intersection/incomings>: incomings of intersection are immutable')
@@ -210,8 +283,6 @@ class Intersection:
             if crossings is None:
                 self._crossings = set()
             else:
-                assert len(crossings) > 0, '<Intersection/crossings>: Intersection crossing ' \
-                                           'must consist of at least one crossing lanelet '
                 self._crossings = crossings
         else:
             warnings.warn('<Intersection/crossings>: crossings of intersection are immutable')
